/*----------------------------------------------------------------------------
 *
 * file   : canvas.c for mag2
 *
 * Author : Yusuf Attarwala
 * Date   : Oct 96
 *
 *---------------------------------------------------------------------------*/

#include <X11/keysym.h>
#include <X11/StringDefs.h>
#include "globals.h"
#include <X11/extensions/readdisplay.h>

extern void drawScene(Picture *pic),
            simpleMesh(Picture *pic),
            editMeshPoint(Picture *pic, int pt, int sx, int sy),
	    highlightPixel(Picture *pic,int x, int y),
	    computeDimension(Picture *pic,int x1, int y1, int x2, int y2);

extern int pickMeshPoint(Picture *pic, int sx, int sy);

static void getRootXY(int *x, int *y),
            setMatrix(Picture *pic),
	    doPan(Picture *pic, float dx, float dy),
	    doReset(Picture *pic),
	    doSnap(Picture *pic),
	    initPicture(Picture *pic),
	    updateTexture(Picture *pic, XImage *img);

void        enableDoubleBuffer(Picture *pic),
            defineTexture(Picture *pic),
            enableSingleBuffer(Picture *pic),
	    initTexture(Picture *pic),
	    updateTextureEnv(Picture *pic),
	    doZoom(Picture *pic, float dx, float dy),
	    doRotate(Picture *pic, float dr),
	    grabImage(Picture *pic,int x,int y);

static int findPowerOfTwo(int);

static void
setMatrix(Picture *pic)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(pic->left,pic->right,pic->bottom,pic->top);
    /*
    printf("setMatrix %f %f %f %f \n",pic->left,pic->right,pic->bottom,pic->top);
    */
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    if (pic->angle != 0.0) {
	glTranslatef((pic->vert[2][0]-pic->vert[0][0])/2.0,
		      (pic->vert[2][1]-pic->vert[3][1])/2.0,
		      0.0);
        glRotatef(pic->angle,0.0,0.0,1.0);
	glTranslatef(-(pic->vert[2][0]-pic->vert[0][0])/2.0,
		     -(pic->vert[2][1]-pic->vert[3][1])/2.0,
		      0.0);
    }
}

void
computeWorldToScreen(Picture *pic)
{
    pic->wsx = (pic->right-pic->left)/(float)pic->width;
    pic->wsy = (pic->top-pic->bottom)/(float)pic->height;
    pic->waspect = (pic->right-pic->left)/(pic->top-pic->bottom);
    pic->gridDelta = 1;
    while ( pic->gridDelta / MAX(pic->wsx,pic->wsy) < 7.0) pic->gridDelta += 1;
}

/* return a number which is >= num and is power of two */
static int
findPowerOfTwo(int num)
{
    int done =0;
    int retVal = 2;

    if (num < 2) return 2;

    while (!done) {
        if (num > retVal) {
            retVal *= 2;
        }
        else {
            done = 1;
        }
    }
    return retVal;
}

void
defineTexture(Picture *pic)
{
    GLwDrawingAreaMakeCurrent(pic->glw, pic->glc);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, pic->dataWidth, pic->dataHeight, 
                 0, GL_RGB, GL_UNSIGNED_BYTE,
                 (const unsigned char*) pic->data);

    /*
    glBindTextureEXT(GL_TEXTURE_2D,7);
    */
    gluErrorString(glGetError());
}

void
updateTexture(Picture *pic, XImage *img)
{
    int i,j,k;
    int w2,h2;
    unsigned char *t;
    long pixval;

    w2 = findPowerOfTwo(img->width);
    h2 = findPowerOfTwo(img->height);

    /*
    printf("w2 %d h2 %d\n",w2,h2);
    */

    if (w2 > pic->dataWidth ||
	h2 > pic->dataHeight) {

	if (pic->data) free((char *)pic->data);

        pic->data = (unsigned char *)malloc(w2*h2*3*sizeof(unsigned char));

	pic->dataWidth  = w2;
	pic->dataHeight = h2;
    }

    pic->imgWidth = img->width;
    pic->imgHeight = img->height;

    k = 0;
    for (j=img->height-1;j>=0;j--,k++) {
        t = pic->data + k*w2*3;
        for (i=0;i<img->width;i++) {
            pixval = XGetPixel(img,i,j);
            *t++ =  pixval & 0x000000ff;
            *t++ = (pixval>>8) & 0x0000ff;
            *t++ = (pixval>>16) & 0x00ff;
	    /*
            *t++ = 255;
	    */
        }
    }

    pic->tex [1][0] = 0.0;                         /* ll */
    pic->tex [1][1] = 0.0;           

    pic->tex [3][0] = (float)img->width/(float)pic->dataWidth; /* lr */
    pic->tex [3][1] = 0.0;           

    pic->tex [2][0] = pic->tex [3][0];             /* ur */
    pic->tex [2][1] = (float)img->height/(float)pic->dataHeight; 

    pic->tex [0][0] = 0.0;               /* ul */
    pic->tex [0][1] = pic->tex [2][1];

    defineTexture(pic);
    simpleMesh(pic);

}

void
updateTextureEnv(Picture *pic)
{
    GLwDrawingAreaMakeCurrent(pic->glw_single, pic->glc_single);
    if (!pic->interpolated) {
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    }
    else {
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    }
    GLwDrawingAreaMakeCurrent(pic->glw_double, pic->glc_double);
    if (!pic->interpolated) {
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    }
    else {
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    }

    if (pic->doubleBuffer) {
        GLwDrawingAreaMakeCurrent(pic->glw_double, pic->glc_double);
    }
    else {
        GLwDrawingAreaMakeCurrent(pic->glw_single, pic->glc_single);
    }
}

void
initTextures(Picture *pic)
{
    GLwDrawingAreaMakeCurrent(pic->glw, pic->glc);

    glPixelStorei(GL_UNPACK_ALIGNMENT,1);

    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

    if (!pic->interpolated) {
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    }
    else {
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    }

    glEnable(GL_TEXTURE_2D);
}

void
doZoom(Picture *pic, float dx, float dy)
{
    pic->top    -= dy;
    pic->bottom += dy;
    pic->left   += dx;
    pic->right  -= dx;

    if (pic->top   <= pic->bottom ||
	pic->right <= pic->left)  {

	pic->top    += dy;
	pic->bottom -= dy;
	pic->left   -= dx;
	pic->right  += dx;

	return;
    }

    setMatrix(pic);
    computeWorldToScreen(pic);
    drawScene(pic);
}

void
doRotate(Picture *pic, float dr)
{
    pic->angle  += dr;

    if (pic->angle > 360.0) 
	pic->angle -= 360.0;
    else if (pic->angle < 0.0) 
	pic->angle += 360.0;

    setMatrix(pic);
    drawScene(pic);
}

static void
doPan(Picture *pic, float dx, float dy)
{
    pic->top    += dy;
    pic->bottom += dy;
    pic->left   += dx;
    pic->right  += dx;

    setMatrix(pic);
    drawScene(pic);
}

static void
doCenter(Picture *pic, int sx, int sy)
{
    float wx,wy;
    float delta;
    screenToWorld(pic,sx,sy,&wx,&wy);

    delta = pic->right - pic->left;
    pic->left = wx - delta/2.0;
    pic->right = pic->left + delta;

    delta = pic->top - pic->bottom;
    pic->bottom = wy - delta/2.0;
    pic->top    = pic->bottom + delta;

    setMatrix(pic);
    drawScene(pic);
}

void panDownCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    doPan(pic,0.0,pic->pixelToPan);
}

void panUpCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    doPan(pic,0.0,-pic->pixelToPan);
}

void panLeftCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    doPan(pic,pic->pixelToPan,0.0);
}

void panRightCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    doPan(pic,-pic->pixelToPan,0.0);
}

void zoomOutCB (Widget w, XtPointer data, XtPointer call)
{
    Picture *pic = (Picture *)data;
    doZoom(pic,-5.0,-(5.0/pic->waspect));

}

void zoomInCB (Widget w, XtPointer data, XtPointer call)
{
    Picture *pic = (Picture *)data;
    doZoom(pic,5.0,(5.0/pic->waspect));

}

void exposeCB (Widget w, XtPointer data, XtPointer call)
{
    Picture *pic = (Picture *)data;
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;

    if (call_data->event->xexpose.count > 0) return;

    pic->width  = call_data->width;
    pic->height = call_data->height;

    /*
    printf("expose %d %d %d\n",w,pic->width,pic->height);
    */

    GLwDrawingAreaMakeCurrent(w, pic->glc);
    glViewport(0, 0, call_data->width, call_data->height);

    drawScene(pic);
}

void initCB (Widget w, XtPointer data, XtPointer call)
{
    Picture *pic = (Picture *)data;
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;

    pic->width  = call_data->width;
    pic->height = call_data->height;

    pic->glc = (w == pic->glw_double) ? pic->glc_double : pic->glc_single;
    pic->glw = w;

    GLwDrawingAreaMakeCurrent(w, (w == pic->glw_double) ? pic->glc_double : pic->glc_single);
    glClearColor(0.0, 0.0, 0.0, 0.0);

    /*
    glDisable(GL_DITHER);
    */

    initPicture(pic);
    initTextures(pic);
    setMatrix(pic);

    XSync(display,True);
}

void
enableDoubleBuffer(Picture *pic)
{
    WidgetList unman,man;
    if (pic->doubleBuffer) return;

    /*
    printf("to double buffer \n");
    */

    pic->glc = pic->glc_double;
    pic->glw = pic->glw_double;
    if (XtIsManaged(frame_single)) {
	unman = &frame_double;
	man   = &frame_single;
	/*
	XtChangeManagedSet(unman,1,man,1,NULL,NULL);
	*/
    /*
    printf("to double 1 \n");
    */
	XtUnmanageChild(frame_single);
        XtManageChild(frame_double);
    }
    pic->doubleBuffer = 1;
    defineTexture(pic);
    /*
    printf("to double 3 \n");
    */
    setMatrix(pic);
    XSync(display,True);
}

void
enableSingleBuffer(Picture *pic)
{
    WidgetList unman,man;
    if (!pic->doubleBuffer) return;

    /*
    printf("to single buffer \n");
    */

    pic->glc = pic->glc_single;
    pic->glw = pic->glw_single;
    if (XtIsManaged(frame_double)) {
	unman = &frame_single;
	man   = &frame_double;
	/*
	XtChangeManagedSet(unman,1,man,1,NULL,NULL);
	*/
	XtUnmanageChild(frame_double);
	XtManageChild(frame_single);
    }
    pic->doubleBuffer = 0;
    defineTexture(pic);
    setMatrix(pic);
}

void inputCB (Widget w, XtPointer data, XtPointer call)
{
    Picture *pic = (Picture *)data;
    char            string[31];
    XComposeStatus  composeStatus;
    KeySym keysym;
    GLwDrawingAreaCallbackStruct *call_data;
    float rl,crossP;
    int cx,cy;
    static int compress = 0;
    static Time oldTime = 0, newTime;
    static int doubleClick = 0;
    static int firstMove = 0;
    static int gridPoint;
    static int xmark1 = -1,ymark1 = -1,xmark2,ymark2;

    static int dx,dy,curX,curY,oldX,oldY;
    static int button1State,button2State,button3State;
    static int shiftKeyPress = 0,ctrlKeyPress = 0;

    call_data = (GLwDrawingAreaCallbackStruct *) call;

    switch(call_data->event->type) {
    case ButtonPress:

	/*
	printf("button press \n");
	*/
	/* check for double clicks */
	newTime = call_data->event->xbutton.time;
	if (newTime-oldTime < 400) {
	    if (ABS(oldX-call_data->event->xbutton.x)<5 &&
		ABS(oldY-call_data->event->xbutton.y)<5)
	    doubleClick = 1;
	}
	else
	    doubleClick = 0;
	oldTime = newTime;
	firstMove = 1;

        oldX = curX = call_data->event->xbutton.x;
        oldY = curY = call_data->event->xbutton.y;
	compress = 0;

        switch (call_data->event->xbutton.button) {
        case Button1:
            button1State = 1;

	    if (shiftKeyPress && pic->editMeshMode) {
	        gridPoint = pickMeshPoint(pic,curX,curY);
	    }
            if (doubleClick) doCenter(pic,oldX,oldY);
	    XDefineCursor(display,XtWindow(pic->glw),panCursor);
            break;
        case Button2 :
            button2State = 1;
	    XDefineCursor(display,XtWindow(pic->glw),zoomCursor);
            break;
        case Button3 :
            button3State = 1;
	    XDefineCursor(display,XtWindow(pic->glw),rotateCursor);
            break;
        }
        break;
    case ButtonRelease:

	/*
	printf("button release \n");
	*/
        oldX = curX = call_data->event->xbutton.x;
        oldY = curY = call_data->event->xbutton.y;

	XUndefineCursor(display,XtWindow(pic->glw));

        switch (call_data->event->xbutton.button) {
        case Button1:
	    if (pic->degenMode) pic->liteWeight = 0;
	    enableSingleBuffer(pic);
	    XUndefineCursor(display,XtWindow(pic->glw));
            button1State = 0;
            break;
        case Button2 :
	    if (pic->degenMode) pic->liteWeight = 0;
	    enableSingleBuffer(pic);
	    XUndefineCursor(display,XtWindow(pic->glw));
            button2State = 0;
            break;
        case Button3 :
	    if (pic->degenMode) pic->liteWeight = 0;
	    enableSingleBuffer(pic);
	    XUndefineCursor(display,XtWindow(pic->glw));
            button3State = 0;
            break;
        }

	button1State = button2State = button3State = 0;

        break;
    case MotionNotify :
	if (firstMove) {
	    firstMove = 0;
	    if (pic->degenMode) pic->liteWeight = 1;
	    enableDoubleBuffer(pic);
            if (button1State)
	        XDefineCursor(display,XtWindow(pic->glw),panCursor);
	    else if (button2State) 
	        XDefineCursor(display,XtWindow(pic->glw),zoomCursor);
	    else if (button3State) 
	        XDefineCursor(display,XtWindow(pic->glw),rotateCursor);
	}
	compress++;
	/*
	if (compress%2) break;
	*/

        dx = oldX - curX;
        dy = oldY - curY;

        oldX = curX;
        oldY = curY;


        curX = call_data->event->xbutton.x;
        curY = call_data->event->xbutton.y;

        if (button1State) {
	    if (shiftKeyPress && pic->editMeshMode) {
	        if (gridPoint != -1) {
		    editMeshPoint(pic,gridPoint,curX,curY);
		    drawScene(pic);
		}
	    }
            else {
	        doPan(pic,pic->wsx*dx,-pic->wsy*dy);
	    }
        }
	else if (button2State) {
	    rl = (ABS(dx) >= ABS(dy)) ? -dx*pic->wsx : dy*pic->wsy;
	    doZoom(pic,rl,(rl/pic->waspect));
	}
	else if (button3State) {
	    cx = pic->width/2.0;
	    cy = pic->height/2.0;

	    if (ABS(dx) >= ABS(dy)) {
		if (curX > oldX) {
		    crossP = (curY >= cy) ? 5.0 : -5.0;
		}
		else  {
		    crossP = (curY >= cy) ? -5.0 : 5.0;
		}
	    }
	    else {
		if (curY > oldY ) {
		    crossP = (curX >= cx) ? -5.0 : 5.0;
		}
		else  {
		    crossP = (curX >= cx) ? 5.0 : -5.0;
		}
	    }
	    doRotate(pic,crossP);
	}

	break;
    case KeyPress :
        XLookupString((XKeyEvent *)call_data->event,string,
                      30,&keysym,&composeStatus);
        switch(keysym) {
	case XK_w :     /* toggle background color */
	    pic->whiteBackground = !pic->whiteBackground;
	    if (pic->whiteBackground) {
		pic->background[0] = 1.0;
		pic->background[1] = 1.0;
		pic->background[2] = 1.0;
	    }
	    else {
		pic->background[0] = 0.0;
		pic->background[1] = 0.0;
		pic->background[2] = 0.0;
	    }
	    drawScene(pic);
	    break;
	/* experimental
	case XK_l :
	    {
	    Dimension w;
	    XtVaGetValues(topmostlevel,XmNwidth,&w,NULL);
	    printf("%d \n",w);
	    XtVaSetValues(topmostlevel,XmNwidth,w+10,NULL);
	    }
	    break;
	*/
	case XK_m :   /* mark pixels for dimensioning */
            if (pic->angle != 0 && pic->angle != 360.0) {
		postError("Currently, dimensioning is only supported \nin Non Rotated mode.\nPlease reset the rotation angle to zero.");
		break;
            }
	    else {
                 Window ignoreWindow,ignoreChild;
                 int rx,ry,x,y;
                 unsigned int mask;
                 XQueryPointer(display,XtWindow(pic->glw),&ignoreWindow,&ignoreChild,
                                 &rx,&ry,&x,&y,&mask);

		 if (xmark1 == -1) {
		     xmark1 = x;
		     ymark1 = y;
		     highlightPixel(pic,xmark1,ymark1);
		 }
		 else {
		     xmark2 = x;
		     ymark2 = y;
		     highlightPixel(pic,xmark2,ymark2);
		     computeDimension(pic,xmark1,ymark1,xmark2,ymark2);
		     xmark1 = -1;
		 }
            }
            break;
	case XK_r :      /* reset */
	    doReset(pic);
	    break;
	case XK_Shift_L :
	case XK_Shift_R :
            shiftKeyPress = 1;
	    break;
	case XK_Left :   /* pan left */
            doPan(pic,pic->pixelToPan,0.0);
	    break;
	case XK_Right :  /* pan right */
            doPan(pic,-pic->pixelToPan,0.0);
	    break;
	case XK_Up :     /* pan up */
            doPan(pic,0.0,-pic->pixelToPan);
	    break;
	case XK_Down :   /* pan down */
            doPan(pic,0.0,pic->pixelToPan);
	    break;
	case XK_i :      /* zoom in */
            doZoom(pic,5.0,(5.0/pic->waspect));
	    break;
	case XK_o :      /* zoom out */
            doZoom(pic,-5.0,-(5.0/pic->waspect));
	    break;
	case XK_p :
	    printf("+\n");
	    doRotate(pic,-1.0);
	    break;
	case XK_minus :
	    doRotate(pic,1.0);
	    break;
	case XK_s :      /* enable snap mode */
	    doSnap(pic);
	    break;
	case XK_e :      /* toggle edit mode */
	    pic->editMeshMode = !pic->editMeshMode;
	    drawScene(pic);
	    break;
        }
        break;
    case KeyRelease :
        XLookupString((XKeyEvent *)call_data->event,string,
                      30,&keysym,&composeStatus);
        switch(keysym) {
        case XK_Escape :
	case XK_q :
	    exit(1);
	case XK_Shift_L :
	case XK_Shift_R :
            shiftKeyPress = 0;
	    break;
        }
        break;
    }
}


void resizeCB (Widget w, XtPointer data, XtPointer call)
{
    Picture *pic = (Picture *)data;
    GLwDrawingAreaCallbackStruct *call_data;
    call_data = (GLwDrawingAreaCallbackStruct *) call;

    /*
    printf("resize %d\n",w);
    */

    pic->width  = call_data->width;
    pic->height = call_data->height;

    computeWorldToScreen(pic);

    GLwDrawingAreaMakeCurrent(w, pic->glc);
    glViewport(0, 0, call_data->width, call_data->height);

}

static void
doReset(Picture *pic)
{

    pic->vert[3][0] = pic->width;
    pic->vert[2][0] = pic->vert[3][0];
    pic->vert[2][1] = pic->height;
    pic->vert[0][1] = pic->vert[2][1];

    pic->left   = 0.0;
    pic->bottom = 0.0;
    pic->right  = pic->vert[3][0];
    pic->top    = pic->vert[2][1];

    pic->angle  = 0.0;

    computeWorldToScreen(pic);
    setMatrix(pic);
    drawScene(pic);
}

void resetCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    doReset(pic);

}

void resetRotateCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    pic->angle = 0.0;
    setMatrix(pic);
    drawScene(pic);
}

void dismissCB (Widget w, XtPointer data)
{
    exit(1);
}

void gridCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    pic->drawGrid = !pic->drawGrid;
    drawScene(pic);
}

void degenCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    pic->degenMode = !pic->degenMode;
}

void interpolateCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    pic->interpolated = !pic->interpolated;
    updateTextureEnv(pic);
    drawScene(pic);
}

void
grabImage(Picture *pic,int x,int y)
{
    XImage *img;
    unsigned long hints_return;

    /*
    printf("grabimage %d %d \n",pic->width,pic->height);
    */

    img = XReadDisplay(display,rootWindow,
		   x-(pic->width/2),y-(pic->height/2),
		   pic->width,pic->height,0,&hints_return);

    updateTexture(pic,img);
    XtFree((char *)img);
}

static void
doSnap(Picture *pic)
{
    int x,y;

    getRootXY(&x,&y);
    grabImage(pic,x,y);
    doReset(pic);
}

void snapButtonCB (Widget w, XtPointer data)
{
    Picture *pic = (Picture *)data;
    doSnap(pic);

}


/* get mouse click on root window */
static void
getRootXY(int *x, int *y)
{

  int status;
  int buttons = 0;
  XEvent event;

  /* Grab the pointer using target cursor */
  status = XGrabPointer(display, rootWindow, False,
                        ButtonPressMask|ButtonReleaseMask, GrabModeSync,
                        GrabModeAsync, rootWindow, magCursor, CurrentTime);

  /* Let the user click on root window */
  while (buttons == 0) {
      /* allow one more event */
      XAllowEvents(display, SyncPointer, CurrentTime);
      XWindowEvent(display, rootWindow, ButtonPressMask|ButtonReleaseMask, &event);
      switch (event.type) {
      case ButtonPress:
          *x = event.xbutton.x;
          *y = event.xbutton.y;
          buttons++;
          break;
      }
  }

  XUngrabPointer(display, CurrentTime);      /* Done with grab mode */
}

static void
initPicture(Picture *pic)
{
    pic->left   = 0.0;
    pic->bottom = 0.0;
    pic->right =  pic->width;
    pic->top   =  pic->height;

    pic->vert[1][0] = pic->left;       /* ll */
    pic->vert[1][1] = pic->bottom;
    pic->tex [1][0] = 0.0;             /* ll */
    pic->tex [1][1] = 0.0;

    pic->vert[3][0] = pic->width;      /* lr */
    pic->vert[3][1] = pic->bottom;
    pic->tex [3][0] = 1.0;             /* lr */
    pic->tex [3][1] = 0.0;

    pic->vert[2][0] = pic->vert[3][0]; /* ur */
    pic->vert[2][1] = pic->height;
    pic->tex [2][0] = 1.0;             /* ur */
    pic->tex [2][1] = 1.0;

    pic->vert[0][0] = pic->vert[1][0]; /* ul */
    pic->vert[0][1] = pic->vert[2][1];
    pic->tex [0][0] = 0.0;             /* ul */
    pic->tex [0][1] = 1.0;

    pic->pixelToPan = 1;
    pic->data       = 0;
    pic->dataWidth  = 0;
    pic->dataHeight = 0;

    pic->angle      = 0.0;
    pic->drawGrid   = 0;

    pic->liteWeight = 0;
    pic->degenMode  = 0;
    pic->interpolated = 0;

    pic->editMeshMode = 0;
    pic->whiteBackground = 0;
    pic->background[0] = 0.0;
    pic->background[1] = 0.0;
    pic->background[2] = 0.0;

    simpleMesh(pic);

    computeWorldToScreen(pic);
}

