/*----------------------------------------------------------------------------
 *
 * file   : cursor.c 
 *
 * Author : Yusuf Attarwala
 * Date   : Oct 96
 *
 *---------------------------------------------------------------------------*/
#include <X11/cursorfont.h>

#include "globals.h"
#include "bitmaps.h"

/* function declarations */
static void createCameraCursor(),
	    createMagCursor(),
            createWindowCursor();
void createCursors();

void 
createCursors()
{
    crossHair    = XCreateFontCursor(display, XC_crosshair);
    busyCursor   = XCreateFontCursor(display, XC_watch);
    panCursor    = XCreateFontCursor(display, XC_fleur);
    zoomCursor   = XCreateFontCursor(display, XC_sizing);
    rotateCursor = XCreateFontCursor(display, XC_exchange);

    createCameraCursor();
    createWindowCursor();
    createMagCursor();

    /* temp */
    {
        Pixel bg,fg;
        XtVaGetValues(toplevel,XmNforeground,&fg,XmNbackground,&bg,NULL);
        if (bg <=50 ) {
            cameraPixmap = XCreatePixmapFromBitmapData(display,rootWindow,
			    ss_bits,ss_width,ss_height,
			    brownColor.pixel,bg,DefaultDepth(display,screen));
        }
	else {
            cameraPixmap = XCreatePixmapFromBitmapData(display,rootWindow,
			    ss_bits,ss_width,ss_height,
			    fg,bg,DefaultDepth(display,screen));
	}
    }
}


static void 
createCameraCursor()
{
    Pixmap pix, mask;
    XColor f, b;
    Colormap ccc;
    pix = XCreatePixmapFromBitmapData(display, rootWindow, 
                  camera_bits, camera_width, camera_height, 
		  1, 0, 1);
    mask = XCreatePixmapFromBitmapData(display,
                  rootWindow, camera_mask_bits, camera_mask_width,
                  camera_mask_height, 1, 0, 1);
    f.red = 244 << 8;
    f.blue = f.green = 0;
    f.flags = DoRed | DoGreen | DoBlue;
    ccc = XDefaultColormap(display, screen);
    XAllocColor(display, ccc, &f);
    b.pixel = 0;

    cameraCursor = XCreatePixmapCursor(display, mask, pix,
        &b, &f, camera_x_hot, camera_y_hot);
}

static void 
createMagCursor()
{
    Pixmap pix, mask;
    XColor f, b;
    Colormap ccc;
    pix = XCreatePixmapFromBitmapData(display, rootWindow, 
                  zoomplus_bits, zoomplus_width, zoomplus_height, 
		  1, 0, 1);
    mask = XCreatePixmapFromBitmapData(display,
                  rootWindow, zoomplusMask_bits, zoomplusMask_width,
                  zoomplusMask_height, 1, 0, 1);
    f.red = 244 << 8;
    f.blue = f.green = 40;
    f.flags = DoRed | DoGreen | DoBlue;
    ccc = XDefaultColormap(display, screen);
    XAllocColor(display, ccc, &f);
    b.pixel = 0;

    magCursor = XCreatePixmapCursor(display, mask, pix,
        &b, &f, zoomplus_x_hot, zoomplus_y_hot);
}

static void 
createWindowCursor()
{
    Pixmap pix, mask;
    XColor f, b;
    Colormap ccc;
    pix = XCreatePixmapFromBitmapData(display,
                  rootWindow, (char*) window_bits, window_width,
                  window_height, 1, 0, 1);
    mask = XCreatePixmapFromBitmapData(display,
                  rootWindow, (char*) window_mask_bits, window_mask_width,
                  window_mask_height, 1, 0, 1);
    f.red = 244 << 8;
    f.blue = f.green = 0;
    f.flags = DoRed | DoGreen | DoBlue;
    ccc = XDefaultColormap(display, screen);
    XAllocColor(display, ccc, &f);
    b.pixel = 0xffffff;

    windowCursor = XCreatePixmapCursor(display, mask, pix,
        &b, &f, window_x_hot, window_y_hot);
}
