/*----------------------------------------------------------------------------
 *
 * file   : draw.c for mag2
 *
 * Author : Yusuf Attarwala
 * Date   : Oct 96
 *
 *---------------------------------------------------------------------------*/

#include <math.h>
#include <X11/keysym.h>
#include <X11/StringDefs.h>
#include "globals.h"
#include <X11/extensions/readdisplay.h>

void drawScene(Picture *pic),
     drawGrid(Picture *pic),
     drawMeshPoints(Picture *pic),
     drawAxes(Picture *pic),
     drawMesh(Picture *pic);


#define XMESH

void
screenToWorld(Picture *pic, int sx, int sy, float *wx, float *wy)
{
    if (pic->angle == 0.0 || pic->angle == 360.0) {
	*wx = (float)(pic->left   + sx * pic->wsx);
	*wy = (float)(pic->bottom + ((float)(pic->height)-sy)*pic->wsy);
    }
    else {
	*wx = 0.0;
	*wy = 0.0;
    }
}

void
drawScene(Picture *pic)
{
    int i;
    glClearColor(pic->background[0],
                 pic->background[1],
                 pic->background[2],
		 1.0);

    glClear(GL_COLOR_BUFFER_BIT);

    /*
    printf("drawscene %f %f (%f %f)\n",pic->vert[2][0],pic->vert[2][1],
				       pic->tex[2][0],pic->tex[2][1]);
    */

    if (pic->liteWeight) {
	if (pic->editMeshMode) {
	    drawMeshPoints(pic);
	}
	else {
	    drawGrid(pic);
	    drawAxes(pic);
	}
    }
    else {
	/* draw the simple tri-strip */

	if (!pic->editMeshMode) {
	    glColor3f(1.0,1.0,1.0);
	    glBegin(GL_TRIANGLE_STRIP);
	    for (i=0;i<4;i++) {
		glTexCoord2fv(pic->tex [i]);
		glVertex2fv  (pic->vert[i]);
	    }
	    glEnd();
	}
	else {
            drawMesh(pic);
	}

	if (pic->drawGrid) {
	    drawGrid(pic);
	    drawAxes(pic);
	}
    }


    if (pic->doubleBuffer) {
        GLwDrawingAreaSwapBuffers(pic->glw);
    }
    else  {
	glFlush();
    }
}

void
drawGrid(Picture *pic)
{
    float maxLimit,minLimit;
    float c;


    if (pic->liteWeight) {
        glColor3f(0.5,0.5,0.5);
    }
    else {
	if (pic->gridDelta > 1.0) return;
        glColor3f(pic->background[0],
                  pic->background[1],
                  pic->background[2]);
    }

    minLimit = pic->vert[1][0];
    maxLimit = pic->vert[3][0];

    glDisable(GL_TEXTURE_2D);
    glBegin(GL_LINES);
    c = minLimit;
    while (c <= maxLimit) {
	glVertex2f(c,pic->vert[1][1]);
	glVertex2f(c,pic->vert[0][1]);
	c += (float)pic->gridDelta;
    }
    minLimit = (int)pic->vert[1][1];
    maxLimit = (int)pic->vert[0][1];
    c = minLimit;
    while (c <= maxLimit) {
	glVertex2f(pic->vert[1][0],c);
	glVertex2f(pic->vert[3][0],c);
	c += (float)pic->gridDelta;
    }
    glEnd();
    glEnable(GL_TEXTURE_2D);

}

void
drawAxes(Picture *pic)
{

    float cx,cy;

    cx = pic->width/2.0;
    cy = pic->height/2.0;

    glDisable(GL_TEXTURE_2D);
    glBegin(GL_LINES);
        glColor3f(1.0,0.0,0.0);
	glVertex2f(pic->vert[1][0],cy);
	glVertex2f(pic->vert[3][0],cy);
        glColor3f(0.0,1.0,0.0);
	glVertex2f(cx,pic->vert[1][1]);
	glVertex2f(cx,pic->vert[0][1]);
    glEnd();
    glEnable(GL_TEXTURE_2D);
}

void
drawMesh(Picture *pic)
{
    int i,j;
    int p;

    glColor3f(1.0,1.0,1.0);

    for (i=0;i<NSTRIP;i++) {
	p = i*(STRIPLEN+1);
	glBegin(GL_TRIANGLE_STRIP);
	for (j=0;j<=STRIPLEN;j++) {
	    glTexCoord2fv(pic->tgrid[p]);
	    glVertex2fv  (pic->vgrid[p]);

	    glTexCoord2fv(pic->tgrid[p+STRIPLEN+1]);
	    glVertex2fv  (pic->vgrid[p+STRIPLEN+1]);

	    p++;
	}
        glEnd();
    }
    drawMeshPoints(pic);
}

void
drawMeshPoints(Picture *pic)
{
    int i,j,p;
    glDisable(GL_TEXTURE_2D);
    glPointSize(2.0);
    glBegin(GL_POINTS);
    p = 0;
    for (i=0;i<=NSTRIP;i++) {
	for (j=0;j<=STRIPLEN;j++) {
	    glVertex2fv  (pic->vgrid[p]);
	    p++;
	}
    }
    glEnd();
    glPointSize(1.0);
    glEnable(GL_TEXTURE_2D);
}

void
highlightPixel(Picture *pic, int sx,int sy)
{
    float wx,wy;
    int   ix,iy;
    screenToWorld(pic,sx,sy,&wx,&wy);
    ix  = (wx > (int)wx) ? (int)wx : (int)(wx) - 1;
    iy  = (wy > (int)wy) ? (int)wy : (int)(wy) - 1;

    glColor3f(pic->background[0],
	      pic->background[1],
	      pic->background[2]);

    glDisable(GL_TEXTURE_2D);
    glBegin(GL_POLYGON);
    glVertex2i(ix,iy);
    glVertex2i(ix+1,iy);
    glVertex2i(ix+1,iy+1);
    glVertex2i(ix,iy+1);
    glEnd();
    glEnable(GL_TEXTURE_2D);
    
    printf("X %d Y %d \n",ix,iy);
}

void
computeDimension(Picture *pic,int x1, int y1, int x2, int y2)
{
    float  wx1,wy1,wx2,wy2;
    int    ix1,iy1,ix2,iy2;

    screenToWorld(pic,x1,y1,&wx1,&wy1);
    screenToWorld(pic,x2,y2,&wx2,&wy2);

    ix1  = (wx1 > (int)wx1) ? (int)wx1 : (int)(wx1) - 1;
    iy1  = (wy1 > (int)wy1) ? (int)wy1 : (int)(wy1) - 1;
    ix2  = (wx2 > (int)wx2) ? (int)wx2 : (int)(wx2) - 1;
    iy2  = (wy2 > (int)wy2) ? (int)wy2 : (int)(wy2) - 1;

    printf("Horizontal span (inclusive) %d \n",1+(ABS(ix2-ix1)));
    printf("Vertical   span (inclusive) %d \n",1+(ABS(iy2-iy1)));
}

