/*----------------------------------------------------------------------------
 *
 * file   : globals.h for mag2
 *
 * Author : Yusuf Attarwala
 * Date   : Oct 96 
 *
 *---------------------------------------------------------------------------*/
#include <stdio.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <GL/gl.h>                  /* gl includes */
#include <GL/glu.h>
#include <GL/GLwMDrawA.h>


#ifdef DEFINE_GLOBALS
#define GLOBAL
#define INIT(x) = x
#else
#define GLOBAL extern
#define INIT(x)
#endif

typedef struct _menuItem {
    Widget             widget;
    char              *label;
    WidgetClass       *class;
    char               mnemonic;
    char              *accelerator;
    char              *accelText;
    void             (*callback)();
    XtPointer          callbackData;
    struct _menuItem  *subItems;    /* rollover menus if not NULL */
} MenuItem;

typedef struct _box2d {
    int x,y;
    Dimension width,height;
} Box2D;

typedef struct _window {
    Window 	window;
    GC          drawgc,
		erasegc,
		draw2gc;
    Box2D       bounds;
} WindowType;

#define NSTRIP   8
#define STRIPLEN 8

typedef struct _picture {
    Widget     glw;
    GLXContext glc;
    Widget     glw_single,glw_double;
    GLXContext glc_single,glc_double;
    int        doubleBuffer;
    Dimension  width,height;
    float      zoom;
    float      angle;
    float      left,right,bottom,top;
    float      wsx,wsy;      /* world/screen ratio */
    float      waspect;
    int        pixelToPan;
    float      vert[4][2];   /* poly vertex   */
    float      tex[4][2];    /* texture coord */

    float      vgrid[(NSTRIP+1)*(STRIPLEN+1)][2];
    float      tgrid[(NSTRIP+1)*(STRIPLEN+1)][2];

    unsigned char *data;
    int        dataWidth;
    int        dataHeight;
    int        imgHeight,imgWidth;

    int        interpolated;
    int        drawGrid,gridDelta;
    int        degenMode;
    int        editMeshMode;
    int        liteWeight;

    float      background[3];
    int        whiteBackground;

} Picture;

#define ABS(a)   (((a) >= 0) ? (a) : -(a))
#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#define MIN(a,b) (((a) < (b)) ? (a) : (b))

GLOBAL Display        *display;
GLOBAL long            screen;
GLOBAL XtAppContext    appContext;
GLOBAL Widget          topmostlevel,toplevel;
GLOBAL Widget          frame_single,frame_double;
GLOBAL Window          rootWindow;
GLOBAL Dimension       screenHeight, screenWidth;


GLOBAL Widget          resetButton,
		       panUpButton,
		       panDownButton,
		       panLeftButton,
		       panRightButton,
		       snapButton,
		       zoomInButton,
		       zoomOutButton;

GLOBAL Widget          helpDialog;
GLOBAL Widget          errDialog;

GLOBAL Widget          gridMenuItem;

GLOBAL Cursor          crossHair,
		       busyCursor,
                       cameraCursor,
		       magCursor,
		       panCursor,
		       zoomCursor,
		       rotateCursor,
		       windowCursor;

GLOBAL XColor          redColor,                  /* some useful colors   */
		       greenColor,
		       blackColor,
		       whiteColor,
		       greyColor,
		       brownColor;

GLOBAL XFontStruct     *mediumNormalFont;
GLOBAL Pixmap          cameraPixmap;
GLOBAL void            screenToWorld(Picture *,int,int,float *,float*);
GLOBAL void            voidFunction();

GLOBAL Picture         canvas;

/* these should be on the last lines */
#undef INIT
#undef GLOBAL
