/*----------------------------------------------------------------------------
 *
 * file   : mesh.c for mag2
 *
 * Author : Yusuf Attarwala
 * Date   : Oct 96
 *
 *---------------------------------------------------------------------------*/

#include <X11/keysym.h>
#include <X11/StringDefs.h>
#include "globals.h"
#include <X11/extensions/readdisplay.h>

void simpleMesh(Picture *pic),
     fishEyeMesh(Picture *pic),
     editMeshPoint(Picture *pic, int pt, int sx, int sy);

int pickMeshPoint(Picture *pic, int sx, int sy);
#define PICK_TOL 20.0

void
simpleMesh(Picture *pic)
{
    int i,j,k;
    float xdelta,ydelta;
    float txdelta,tydelta;
    float p,q;
    float tp,tq;

/*
printf("-- %f %f %f %f \n",pic->vert[3][0],pic->vert[1][0],pic->vert[0][1],pic->vert[1][1]);
*/
    xdelta = (pic->vert[3][0] - pic->vert[1][0]) / (float)STRIPLEN;
    ydelta = (pic->vert[0][1] - pic->vert[1][1]) / (float)NSTRIP;

    txdelta = (pic->tex[3][0] - pic->tex[1][0])/(float)STRIPLEN;
    tydelta = (pic->tex[0][1] - pic->tex[1][1])/(float)NSTRIP;

    k = 0;
    q = pic->vert[0][1];
    tq = pic->tex[0][1];
    for (i=0;i<=NSTRIP;i++) {
	p = pic->vert[1][0];
	tp = pic->tex[1][0];
	for (j=0;j<=STRIPLEN;j++) {
	    pic->vgrid[k][0] = p;
	    pic->vgrid[k][1] = q;
	    pic->tgrid[k][0] = tp;
	    pic->tgrid[k][1] = tq;

	    p += xdelta;
	    tp += txdelta;
	    k++;
	}
	q -= ydelta;
	tq -= tydelta;
    }
}

void
fishEyeMesh(Picture *pic)
{
    int i,j,k;
    float xdelta,ydelta;
    float txdelta,tydelta;
    float p,q;
    float tp,tq;

    xdelta = (pic->vert[3][0] - pic->vert[1][0]) / (float)STRIPLEN;
    ydelta = (pic->vert[0][1] - pic->vert[1][1]) / (float)NSTRIP;

    txdelta = (pic->tex[3][0] - pic->tex[1][0])/(float)STRIPLEN;
    tydelta = (pic->tex[0][1] - pic->tex[1][1])/(float)NSTRIP;

    k = 0;
    q = pic->vert[0][1];
    tq = pic->tex[0][1];
    for (i=0;i<=NSTRIP;i++) {
	p = pic->vert[1][0];
	tp = pic->tex[1][0];
	for (j=0;j<=STRIPLEN;j++) {
	    pic->vgrid[k][0] = p;
	    pic->vgrid[k][1] = q;
	    pic->tgrid[k][0] = tp;
	    pic->tgrid[k][1] = tq;

	    p += xdelta;
	    tp += txdelta;
	    k++;
	}
	q -= ydelta;
	tq -= tydelta;
    }

    pic->vgrid[6][1] += 40.0;
    pic->vgrid[7][1] += 40.0;
    pic->vgrid[8][1] += 40.0;
    pic->vgrid[6][0] -= 40.0;
    pic->vgrid[11][0] -= 40.0;
    pic->vgrid[16][0] -= 40.0;
    pic->vgrid[16][1] -= 40.0;
    pic->vgrid[17][1] -= 40.0;
    pic->vgrid[18][1] -= 40.0;
    pic->vgrid[8][0] += 40.0;
    pic->vgrid[13][0] += 40.0;
    pic->vgrid[18][0] += 40.0;
}

void
editMeshPoint(Picture *pic, int pt, int sx, int sy)
{
    screenToWorld(pic,sx,sy,&pic->vgrid[pt][0],&pic->vgrid[pt][1]);
}

int
pickMeshPoint(Picture *pic, int sx, int sy)
{
    int i,j,p;
    float wx,wy;

    screenToWorld(pic,sx,sy,&wx,&wy);

    p = 0;
    for (i=0;i<=NSTRIP;i++) {
	for (j=0;j<=STRIPLEN;j++) {
	    if (ABS(wx-pic->vgrid[p][0]) < PICK_TOL &&
                ABS(wy-pic->vgrid[p][1]) < PICK_TOL) {
		    /*
		    printf("picked %d \n",p);
		    */
		    return p;
	    }
	    p++;
	}
    }
    return -1;
}

