/*----------------------------------------------------------------------------
 * 
 * file   : ui.c 
 * 
 * Author : Yusuf Attarwala
 * Date   : Sep 96
 * 
 *---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>

#include <Xm/ArrowB.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/MessageB.h>
#include <Xm/MenuShell.h>
#include <Xm/PanedW.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/RowColumn.h>
#include <Xm/SeparatoG.h>
#include <Xm/Separator.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>

#include "globals.h"
#include "bitmaps.h"
#include "callback.h"

static Arg args[40];
static int argcnt;

extern Widget buildMenu();
extern Widget createHelpDialog(Widget parent),
              createMessageWidget(Widget parent);

static Widget createMenuBar(),
              createGraphicsArea(),
	      createControlsArea();

static void createSingleBufferGL(Picture *pic, Widget parent);
static void createDoubleBufferGL(Picture *pic, Widget parent);

#define INIT_HEIGHT 240
#define INIT_WIDTH  350
void updateMenus();

void 
createUI()
{
    Widget form,mb,ga,ca;
    helpDialog    = createHelpDialog(topmostlevel);
    errDialog     = createMessageWidget(topmostlevel);

    form = XtVaCreateManagedWidget("mainform",
				 xmFormWidgetClass,topmostlevel,
				 XmNspacing,     0,
				 XmNmarginHeight,0,
				 XmNmarginWidth, 0,
				 NULL);

    mb = createMenuBar(form,"menub");
    ga = createGraphicsArea(form,"graphicsarea");
    ca = createControlsArea(form,"controlarea");

    XtVaSetValues(mb, XmNtopAttachment,       XmATTACH_FORM,
		      XmNleftAttachment,      XmATTACH_FORM,
		      XmNrightAttachment,     XmATTACH_FORM,
		      NULL);

    XtVaSetValues(ga, XmNtopAttachment,       XmATTACH_WIDGET,
		      XmNtopWidget,           mb,
		      XmNleftAttachment,      XmATTACH_FORM,
		      XmNrightAttachment,     XmATTACH_FORM,
		      XmNbottomAttachment,    XmATTACH_WIDGET,
		      XmNbottomWidget,        ca,
		      NULL);

    XtVaSetValues(ca, XmNleftAttachment,      XmATTACH_FORM,
		      XmNrightAttachment,     XmATTACH_FORM,
		      XmNbottomAttachment,    XmATTACH_FORM,
		      NULL);

    XtRealizeWidget(topmostlevel);
}


static Widget
createMenuBar(parent,name)
Widget parent;
char *name;
{

    Widget menub;
    static MenuItem file_items[] = {
        {NULL,"Exit",&xmPushButtonGadgetClass,'E',"Ctrl<Key>C","Ctrl+C",
             doExitCB,(XtPointer)NULL,(MenuItem *)NULL},

         NULL,
    };

    static MenuItem view_items[] = {
        {NULL,"Grid",&xmToggleButtonGadgetClass,'G',"Ctrl<Key>G","Ctrl+G",
             gridCB,(XtPointer)&canvas,(MenuItem *)NULL},
        {NULL,"Interactive Move",&xmToggleButtonGadgetClass,'I',NULL,NULL,
             degenCB,(XtPointer)&canvas,(MenuItem *)NULL},
        {NULL,"Interpolated",&xmToggleButtonGadgetClass,'n',NULL,NULL,
             interpolateCB,(XtPointer)&canvas,(MenuItem *)NULL},
        {NULL,"Reset Rotation",&xmPushButtonGadgetClass,'R',NULL,NULL,
             resetRotateCB,(XtPointer)&canvas,(MenuItem *)NULL},

         NULL,
    };

    static MenuItem filter_items[] = {
        {NULL,"Invert",&xmPushButtonGadgetClass,'I',NULL,NULL,
             invertFilterCB,(XtPointer)&canvas,(MenuItem *)NULL},

         NULL,
    };

    static MenuItem help_items[] = {
        {NULL,"Overview...",&xmPushButtonGadgetClass,'O',NULL,NULL,
             helpCB,(XtPointer)NULL,(MenuItem *)NULL},

         NULL,
    };

    argcnt = 0;
    XtSetArg(args[argcnt],XmNmarginHeight,0);argcnt++;
    menub = XmCreateMenuBar(parent,name,args,argcnt);
    XtManageChild(menub);

    buildMenu(menub,XmMENU_PULLDOWN,"File", 'F',file_items);
    buildMenu(menub,XmMENU_PULLDOWN,"View", 'V',view_items);
    buildMenu(menub,XmMENU_PULLDOWN,"Filters", 'i',filter_items);
    buildMenu(menub,XmMENU_PULLDOWN,"Help", 'H',help_items);

    {
        /* identify help menu bar */
        Widget *cascadeBtns;
        int nBtns;
        argcnt = 0;
        XtSetArg(args[argcnt],XmNchildren,&cascadeBtns);argcnt++;
        XtSetArg(args[argcnt],XmNnumChildren,&nBtns);argcnt++;
        XtGetValues(menub,args,argcnt);

        XtVaSetValues(menub,XmNmenuHelpWidget,cascadeBtns[nBtns-1],NULL);

    }


    return(menub);
}

static void
createDoubleBufferGL(Picture *pic, Widget parent)
{
    XVisualInfo *vi;

    /* create a double buffered openGL widget */
    pic->glw_double = XtVaCreateManagedWidget("glwidget",
					glwMDrawingAreaWidgetClass, parent,
					GLwNdoublebuffer,       True,
					GLwNrgba,               True,
					GLwNallocateBackground, True,
					XmNbackground,          blackColor.pixel,
					NULL);

    /* register gl callbacks */

    XtAddCallback(pic->glw_double,GLwNginitCallback, (XtCallbackProc)initCB,  
	    (XtPointer)&canvas);
    XtAddCallback(pic->glw_double,GLwNinputCallback, (XtCallbackProc)inputCB, 
	    (XtPointer)&canvas);
    XtAddCallback(pic->glw_double,GLwNresizeCallback,(XtCallbackProc)resizeCB,
	    (XtPointer)&canvas);
    XtAddCallback(pic->glw_double,GLwNexposeCallback,(XtCallbackProc)exposeCB,
	    (XtPointer)&canvas);

    XtVaGetValues(pic->glw_double, GLwNvisualInfo, &vi, NULL);
    /* put the single buffer glc on the shared list */
    pic->glc_double = glXCreateContext(display, vi, pic->glc_single, GL_TRUE);

}

static void
createSingleBufferGL(Picture *pic, Widget parent)
{
    XVisualInfo *vi;

    /* create a single buffered openGL widget */
    pic->glw_single = XtVaCreateManagedWidget("glwidget",
					glwMDrawingAreaWidgetClass, parent,
					GLwNdoublebuffer,       False,
					GLwNrgba,               True,
					GLwNallocateBackground, True,
					XmNbackground,          blackColor.pixel,
					NULL);

    /* register gl callbacks */

    XtAddCallback(pic->glw_single,GLwNginitCallback, (XtCallbackProc)initCB,  
	    (XtPointer)&canvas);
    XtAddCallback(pic->glw_single,GLwNinputCallback, (XtCallbackProc)inputCB, 
	    (XtPointer)&canvas);
    XtAddCallback(pic->glw_single,GLwNresizeCallback,(XtCallbackProc)resizeCB,
	    (XtPointer)&canvas);
    XtAddCallback(pic->glw_single,GLwNexposeCallback,(XtCallbackProc)exposeCB,
	    (XtPointer)&canvas);

    XtVaGetValues(pic->glw_single, GLwNvisualInfo, &vi, NULL);
    /* put the double buffer glc on the shared list */
    pic->glc_single = glXCreateContext(display, vi, pic->glc_double, GL_TRUE);

}

static Widget
createGraphicsArea(parent,name)
Widget parent;
char *name;
{
    Widget form;
    form = XtVaCreateManagedWidget("form",
				    xmFormWidgetClass,      parent,
				    XmNmarginHeight,        0,
				    XmNmarginWidth,         0,
				    XmNheight,              INIT_HEIGHT,
				    XmNwidth,               INIT_WIDTH,
				    NULL);

    frame_double =  XtVaCreateManagedWidget("frame",
				        xmFrameWidgetClass,     form,
					XmNmarginHeight,        0,
					XmNmarginWidth,         0,
					XmNheight,              INIT_HEIGHT,
					XmNwidth,               INIT_WIDTH,
                                        XmNtopAttachment,       XmATTACH_FORM,
				        XmNbottomAttachment,    XmATTACH_FORM,
				        XmNleftAttachment,      XmATTACH_FORM,
				        XmNrightAttachment,     XmATTACH_FORM,
					NULL);

    createDoubleBufferGL(&canvas,frame_double);

    /* also create a single buffer widget */
    /* one of the frames will be remain unmanaged */

    frame_single    = XtVaCreateManagedWidget("frame",
				        xmFrameWidgetClass,     form,
					XmNmarginHeight,        0,
					XmNmarginWidth,         0,
					XmNheight,              INIT_HEIGHT,
					XmNwidth,               INIT_WIDTH,
                                        XmNtopAttachment,       XmATTACH_FORM,
				        XmNbottomAttachment,    XmATTACH_FORM,
				        XmNleftAttachment,      XmATTACH_FORM,
				        XmNrightAttachment,     XmATTACH_FORM,
					NULL);

    createSingleBufferGL(&canvas,frame_single);

    return(form);
}

static Widget
createControlsArea(parent,name)
Widget parent;
char *name;
{
    Widget form,bottomLeftRC,bottomRightRC;
    Pixel fg,bg;
    Pixmap pix;

    form              = XtVaCreateManagedWidget("form",
				        xmFormWidgetClass,      parent,
					NULL);

    /*--------------- bottom left RC ----------------*/

    bottomLeftRC = XtVaCreateManagedWidget("lowerRC",
				 xmRowColumnWidgetClass, form,
				 XmNorientation,         XmHORIZONTAL,
                                 XmNtopAttachment,       XmATTACH_FORM,
				 XmNtopOffset,           5,
				 XmNtopPosition,         0,
				 XmNbottomAttachment,    XmATTACH_FORM,
				 XmNbottomPosition,      0,
				 XmNbottomOffset,        2,
				 XmNleftAttachment,      XmATTACH_FORM,
				 XmNleftOffset,          5,
				 XmNrightAttachment,     XmATTACH_NONE,
				 XmNrightPosition,       0,
				 XmNrightOffset,         0,
				 XmNheight,              30,
				 NULL);

    panLeftButton = XtVaCreateManagedWidget("leftArrow",
					     xmArrowButtonWidgetClass, bottomLeftRC,
					     XmNarrowDirection,        XmARROW_LEFT,
					     NULL);

    XtAddCallback(panLeftButton,XmNactivateCallback,(XtCallbackProc)panLeftCB,
	    (XtPointer)&canvas);
    panRightButton = XtVaCreateManagedWidget("rightArrow",
					     xmArrowButtonWidgetClass, bottomLeftRC,
					     XmNarrowDirection,        XmARROW_RIGHT,
					     NULL);

    XtAddCallback(panRightButton,XmNactivateCallback,(XtCallbackProc)panRightCB,
	    (XtPointer)&canvas);
    panUpButton = XtVaCreateManagedWidget("upArrow",
					     xmArrowButtonWidgetClass, bottomLeftRC,
					     XmNarrowDirection,        XmARROW_UP,
					     NULL);

    XtAddCallback(panUpButton,XmNactivateCallback,(XtCallbackProc)panUpCB,
	    (XtPointer)&canvas);
    panDownButton = XtVaCreateManagedWidget("downArrow",
					     xmArrowButtonWidgetClass, bottomLeftRC,
					     XmNarrowDirection,        XmARROW_DOWN,
					     NULL);

    XtAddCallback(panDownButton,XmNactivateCallback,(XtCallbackProc)panDownCB,
	    (XtPointer)&canvas);

    XtVaGetValues(panDownButton,XmNforeground,&fg,XmNbackground,&bg,NULL);

    pix = XCreatePixmapFromBitmapData(display,rootWindow,
				      zoomminus_bits,zoomminus_width,zoomminus_height,
				      fg,bg,DefaultDepth(display,screen));

    zoomOutButton  = XtVaCreateManagedWidget("z",
					     xmPushButtonWidgetClass, bottomLeftRC,
					     XmNlabelType,            XmPIXMAP,
					     XmNlabelPixmap,          pix,
					     XmNmarginHeight,         0,
					     XmNmarginWidth,          0,
					     XmNbackground,           bg,
					     NULL);

    XtAddCallback(zoomOutButton,XmNactivateCallback,(XtCallbackProc)zoomOutCB,
	    (XtPointer)&canvas);

    pix = XCreatePixmapFromBitmapData(display,rootWindow,
				      zoomplus_bits,zoomplus_width,zoomplus_height,
				      fg,bg,DefaultDepth(display,screen));

    zoomInButton  = XtVaCreateManagedWidget("Z",
					     xmPushButtonWidgetClass, bottomLeftRC,
					     XmNlabelType,            XmPIXMAP,
					     XmNlabelPixmap,          pix,
					     XmNmarginHeight,         0,
					     XmNmarginWidth,          0,
					     XmNbackground,           bg,
					     NULL);

    XtAddCallback(zoomInButton,XmNactivateCallback,(XtCallbackProc)zoomInCB,
	    (XtPointer)&canvas);

    /*--------------- bottom right RC ----------------*/


    bottomRightRC = XtVaCreateManagedWidget("bottomRightRC",
                                       xmRowColumnWidgetClass, form,
				       XmNorientation,         XmHORIZONTAL,
                                       XmNtopAttachment,       XmATTACH_FORM,
				       XmNtopOffset,           0,
				       XmNtopPosition,         0,
                                       XmNbottomAttachment,    XmATTACH_FORM,
				       XmNbottomPosition,      0,
				       XmNbottomOffset,        2,
                                       XmNleftAttachment,      XmATTACH_NONE,
				       XmNleftOffset,          0,
				       XmNleftPosition,        0,
                                       XmNrightAttachment,     XmATTACH_FORM,
                                       XmNrightOffset,         5,
                                       XmNrightPosition,       0,
				       XmNheight,              30,
                                       (XtPointer) NULL );


    resetButton   = XtVaCreateManagedWidget("Reset",
                                         xmPushButtonWidgetClass, bottomRightRC,
                                         XmNlabelType, XmSTRING,
                                         XmNrecomputeSize, True,
                                         (XtPointer) NULL );

    XtAddCallback (resetButton, XmNactivateCallback, (XtCallbackProc)resetCB,
	(XtPointer)&canvas);

    snapButton    = XtVaCreateManagedWidget("Snap",
                                         xmPushButtonWidgetClass, bottomRightRC,
                                         XmNlabelType, XmSTRING,
                                         XmNrecomputeSize, True,
                                         (XtPointer) NULL );
    XtAddCallback (snapButton, XmNactivateCallback, 
	(XtCallbackProc)snapButtonCB, (XtPointer)&canvas);

    return(form);
}


Widget
createMessageWidget(parent)
Widget parent;
{
    /* creates a generic message dialog for frequent use */
    Widget dialog;
    XmString mtext;
    dialog = XmCreateMessageDialog(parent,"message",NULL,0);

    mtext = XmStringCreateSimple("mag2 message");
    XtVaSetValues(dialog,
        XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
        XmNdeleteResponse,      XmDO_NOTHING,
        XmNtitle,"mag2 message",
        XmNdialogTitle,mtext,
        NULL);

    XtAddCallback(dialog,XmNokCallback,(XtCallbackProc)XtUnmanageChild,dialog);
    XtUnmanageChild (XmMessageBoxGetChild(dialog,XmDIALOG_HELP_BUTTON));
    XtUnmanageChild (XmMessageBoxGetChild(dialog,XmDIALOG_CANCEL_BUTTON));
    XmStringFree(mtext);
    return(dialog);
}

void
postError(msg)
char *msg;
{
    XmString text;

    text  = XmStringCreateLtoR(msg,XmSTRING_DEFAULT_CHARSET);
    XtVaSetValues(errDialog,
        XmNmessageString, text,
        NULL);
    XmStringFree(text);
    XtManageChild(errDialog);
}


void
doExitCB(w,data)
Widget w;
XtPointer data;
{
    exit(1);
}

/* just a placeholder */
void voidFunction(){}

